#include "input_variables.h"
#include "histogram_definition.h"

double massFromPID(int pid)
{
  double mass = -999.;
  if(TMath::Abs(pid) == 211) mass = 0.139570;
  else if(TMath::Abs(pid) == 321) mass = 0.493677;
  else if(TMath::Abs(pid) == 310) mass = 0.497614;
  else if(TMath::Abs(pid) == 2212) mass = 0.938272;
  else if(TMath::Abs(pid) == 3122) mass = 1.115683;
  else mass = -999.;
  return mass;
}

void signal_corr(std::vector<float> pthatw_vec, std::vector<int> processID_vec, std::vector<std::vector<TVector3>> jet_vec, std::vector<std::vector<int>> jet_flavour_vec, std::vector<std::vector<std::vector<TVector3>>> jet_oldtrk_vec, std::vector<std::vector<std::vector<TVector3>>> jet_newtrk_vec, std::vector<std::vector<std::vector<TVector3>>> jet_oldchtrk_vec, std::vector<std::vector<std::vector<TVector3>>> jet_newchtrk_vec, std::vector<std::vector<std::vector<int>>> jet_trk_charge, std::vector<std::vector<std::vector<int>>> jet_trk_id)
{
  if(jet_newtrk_vec.size() != jet_vec.size() || jet_newtrk_vec.size() != processID_vec.size())
    {
      std::cout<<"event numbers are not same, pleaes check"<<std::endl;
    }
  
  else std::cout<<"Total events is: "<<jet_newtrk_vec.size()<<"  "<<jet_vec.size()<<"  "<<processID_vec.size()<<std::endl;

  std::cout<<endl;
  std::cout<<"~~~~~start signal correlation~~~~~~~~"<<std::endl;
  
  for(int ievt = 0; ievt < (int)jet_newtrk_vec.size(); ievt++) // event loop
    {
      if(ievt%10000 == 0) std::cout<<ievt<<"  events running for signal correlation of total events "<< jet_newtrk_vec.size() <<std::endl;
      
      double pthatw = pthatw_vec[ievt];

      int subprocessID = processID_vec[ievt];

      int njets = 0;
      
      for(int ijet = 0; ijet < (int)jet_newtrk_vec[ievt].size(); ijet++) // jet loop
	{
	  TVector3 jetVec = jet_vec[ievt][ijet];
	  double jet_pt = jetVec.Pt();
	  double jet_eta = jetVec.Eta();
	  double jet_phi = jetVec.Phi();
	  int jet_flavour = jet_flavour_vec[ievt][ijet];
	  int ntrk_jet = jet_newtrk_vec[ievt][ijet].size();
	  int nchtrk_jet = jet_newchtrk_vec[ievt][ijet].size();

	  // extract the nchtrk bin
	  int nchbin  = hnch_bin_hist->FindBin(nchtrk_jet) - 1;


	  //fill jet sparse histograms
	  double Jet_pT_Eta_Phi_Flv[4] = {jet_pt, jet_eta, jet_phi, (double)jet_flavour};
	  hJet_pT_Eta_Phi_Flv->Fill(Jet_pT_Eta_Phi_Flv, pthatw);

	  //apply jet pt, eta cut
	  if(jet_pt < jet_pt_min || jet_pt > jet_pt_max || TMath::Abs(jet_eta) > jet_eta_max) continue;

	  // extract the jet pt bin
	  int jetptbin  = hjetpt_bin_hist->FindBin(jet_pt) - 1;

	  //std::cout<<jet_pt<<"  "<<jet_flavour<<std::endl;
	  //fill jet histograms
	  hjet_pt->Fill(jet_pt, pthatw);
	  hjet_eta->Fill(jet_eta, pthatw);
	  hjet_phi->Fill(jet_phi, pthatw);
	  hjet_flavour->Fill(jet_flavour, pthatw);
	  hJet_ntrk->Fill(ntrk_jet, pthatw);
	  hJet_nchtrk->Fill(nchtrk_jet, pthatw);
	  hJet_nchtrk_pT->Fill(nchtrk_jet, jet_pt, pthatw);
	  hJet_nchtrk_[nchbin]->Fill(nchtrk_jet, pthatw);
	  
	  
	  for(int itrk = 0; itrk < (int)jet_newtrk_vec[ievt][ijet].size(); itrk++) // trigger trk loop
	    {
	      TVector3 old_trgtrk_vec = jet_oldtrk_vec[ievt][ijet][itrk];
	      double old_trgtrk_pt = old_trgtrk_vec.Pt();
	      double old_trgtrk_eta = old_trgtrk_vec.Eta();
	      double old_trgtrk_phi = old_trgtrk_vec.Phi();
	      TVector3 new_trgtrk_vec = jet_newtrk_vec[ievt][ijet][itrk];
	      double new_trgtrk_pt = new_trgtrk_vec.Pt();
	      double new_trgtrk_eta = new_trgtrk_vec.Eta();
	      double new_trgtrk_phi = new_trgtrk_vec.Phi();
	      int trgtrk_charge = jet_trk_charge[ievt][ijet][itrk];
	      int trgtrk_id = jet_trk_id[ievt][ijet][itrk];

	      double Trk_pT_Eta_Phi_Flv[5] = {old_trgtrk_pt, old_trgtrk_eta, old_trgtrk_phi, (double)jetptbin, (double)jet_flavour};
	      double Trk_pT_Eta_Phi_Flv_pos[5] = {old_trgtrk_pt, old_trgtrk_eta, old_trgtrk_phi, (double)jetptbin, (double)jet_flavour};
	      double Trk_pT_Eta_Phi_Flv_neg[5] = {old_trgtrk_pt, old_trgtrk_eta, old_trgtrk_phi, (double)jetptbin, (double)jet_flavour};
	      double NewTrk_pT_Eta_Phi_Flv[5] = {new_trgtrk_pt, new_trgtrk_eta, new_trgtrk_phi, (double)jetptbin, (double)jet_flavour};
	      double NewTrk_pT_Eta_Phi_Flv_pos[5] = {new_trgtrk_pt, new_trgtrk_eta, new_trgtrk_phi, (double)jetptbin, (double)jet_flavour};
	      double NewTrk_pT_Eta_Phi_Flv_neg[5] = {new_trgtrk_pt, new_trgtrk_eta, new_trgtrk_phi, (double)jetptbin, (double)jet_flavour};


	      if(TMath::Abs(trgtrk_charge) > 0 )
		{
		  hTrk_pT_Eta_Phi_Flv->Fill(Trk_pT_Eta_Phi_Flv, pthatw);
		  hNewTrk_pT_Eta_Phi_Flv->Fill(NewTrk_pT_Eta_Phi_Flv, pthatw);
		  
		  if(trgtrk_charge > 0)
		    {
		      hTrk_pT_Eta_Phi_Flv_pos->Fill(Trk_pT_Eta_Phi_Flv, pthatw);
		      hNewTrk_pT_Eta_Phi_Flv_pos->Fill(NewTrk_pT_Eta_Phi_Flv, pthatw);
		    }
		  if(trgtrk_charge < 0)
		    {
		      hTrk_pT_Eta_Phi_Flv_neg->Fill(Trk_pT_Eta_Phi_Flv, pthatw);
		      hNewTrk_pT_Eta_Phi_Flv_neg->Fill(NewTrk_pT_Eta_Phi_Flv, pthatw);
		    }

		  if(TMath::Abs(new_trgtrk_eta) <= newtrk_eta_max)
		    {
		      if(new_trgtrk_pt > 0.)
			{
			  hchtrk_pt->Fill(old_trgtrk_pt, pthatw);
			  hchtrk_eta->Fill(old_trgtrk_eta, pthatw);
			  hchtrk_phi->Fill(old_trgtrk_phi, pthatw);
			  hchtrk_pt_jetaxis->Fill(new_trgtrk_pt, pthatw);
			  hchtrk_eta_jetaxis->Fill(new_trgtrk_eta, pthatw);
			  hchtrk_phi_jetaxis->Fill(new_trgtrk_phi, pthatw);
			  
			  double NewTrk_Ntrk[3] = {1, (double)jetptbin, (double)jet_flavour};
			  hNewTrk_Ntrk->Fill(NewTrk_Ntrk, pthatw);
			  
			  if(trgtrk_charge > 0)
			    {
			      hchtrk_pt_p->Fill(old_trgtrk_pt, pthatw);
			      hchtrk_eta_p->Fill(old_trgtrk_eta, pthatw);
			      hchtrk_phi_p->Fill(old_trgtrk_phi, pthatw);
			      hchtrk_pt_jetaxis_p->Fill(new_trgtrk_pt, pthatw);
			      hchtrk_eta_jetaxis_p->Fill(new_trgtrk_eta, pthatw);
			      hchtrk_phi_jetaxis_p->Fill(new_trgtrk_phi, pthatw);
			      
			      hNewTrk_Ntrk_p->Fill(NewTrk_Ntrk, pthatw);
			    }
			  if(trgtrk_charge < 0)
			    {
			      hchtrk_pt_m->Fill(old_trgtrk_pt, pthatw);
			      hchtrk_eta_m->Fill(old_trgtrk_eta, pthatw);
			      hchtrk_phi_m->Fill(old_trgtrk_phi, pthatw);
			      hchtrk_pt_jetaxis_m->Fill(new_trgtrk_pt, pthatw);
			      hchtrk_eta_jetaxis_m->Fill(new_trgtrk_eta, pthatw);
			      hchtrk_phi_jetaxis_m->Fill(new_trgtrk_phi, pthatw);
			      
			      hNewTrk_Ntrk_m->Fill(NewTrk_Ntrk, pthatw);
			    }
			} // pt condition
		    } // eta condition
		} // charge condition
	      
	      /*
		if(TMath::Abs(new_trgtrk_eta) <= newtrk_eta_max)
		{
		  halltrk_pt->Fill(old_trgtrk_pt, pthatw);
		  halltrk_eta->Fill(old_trgtrk_eta, pthatw);
		  halltrk_phi->Fill(old_trgtrk_phi, pthatw);
		  halltrk_pt_jetaxis->Fill(new_trgtrk_pt, pthatw);
		  halltrk_eta_jetaxis->Fill(new_trgtrk_eta, pthatw);
		  halltrk_phi_jetaxis->Fill(new_trgtrk_phi, pthatw);

		  if(TMath::Abs(trgtrk_id) == 211) //pion
		    {
		      hpion_pt->Fill(old_trgtrk_pt, pthatw);
		      hpion_eta->Fill(old_trgtrk_eta, pthatw);
		      hpion_phi->Fill(old_trgtrk_phi, pthatw);
		      hpion_pt_jetaxis->Fill(new_trgtrk_pt, pthatw);
		      hpion_eta_jetaxis->Fill(new_trgtrk_eta, pthatw);
		      hpion_phi_jetaxis->Fill(new_trgtrk_phi, pthatw);

		      if(trgtrk_id == 211)
			{
			  hpion_pt_p->Fill(old_trgtrk_pt, pthatw);
			  hpion_eta_p->Fill(old_trgtrk_eta, pthatw);
			  hpion_phi_p->Fill(old_trgtrk_phi, pthatw);
			  hpion_pt_jetaxis_p->Fill(new_trgtrk_pt, pthatw);
			  hpion_eta_jetaxis_p->Fill(new_trgtrk_eta, pthatw);
			  hpion_phi_jetaxis_p->Fill(new_trgtrk_phi, pthatw);
			}
		      else if(trgtrk_id == -211)
			{
			  hpion_pt_m->Fill(old_trgtrk_pt, pthatw);
			  hpion_eta_m->Fill(old_trgtrk_eta, pthatw);
			  hpion_phi_m->Fill(old_trgtrk_phi, pthatw);
			  hpion_pt_jetaxis_m->Fill(new_trgtrk_pt, pthatw);
			  hpion_eta_jetaxis_m->Fill(new_trgtrk_eta, pthatw);
			  hpion_phi_jetaxis_m->Fill(new_trgtrk_phi, pthatw);
			}

		      if(new_trgtrk_pt >= newtrk_jt03_min && new_trgtrk_pt <= newtrk_jt03_max)
			{
			  hnpion_jt03->AddBinContent(1, pthatw);
			  hnpion_jt03_[nchbin]->AddBinContent(1, pthatw);

			  if(trgtrk_id == 211)
			    {
			      hnpion_jt03_pm->AddBinContent(1, pthatw);
			      hnpion_jt03_pm_[nchbin]->AddBinContent(1, pthatw);
			    }
			  else if(trgtrk_id == -211)
			    {
			      hnpion_jt03_pm->AddBinContent(2, pthatw);
			      hnpion_jt03_pm_[nchbin]->AddBinContent(2, pthatw);
			    }
			}
		      if(new_trgtrk_pt >= newtrk_jt0p33_min && new_trgtrk_pt <= newtrk_jt0p33_max)
			{
			  hnpion_jt0p33->AddBinContent(1, pthatw);
                          hnpion_jt0p33_[nchbin]->AddBinContent(1, pthatw);

			  if(trgtrk_id == 211)
                            {
                              hnpion_jt0p33_pm->AddBinContent(1, pthatw);
                              hnpion_jt0p33_pm_[nchbin]->AddBinContent(1, pthatw);
                            }
                          else if(trgtrk_id == -211)
                            {
                              hnpion_jt0p33_pm->AddBinContent(2, pthatw);
                              hnpion_jt0p33_pm_[nchbin]->AddBinContent(2, pthatw);
			    }

			}
		      if(new_trgtrk_pt >= newtrk_jt0p53_min && new_trgtrk_pt <= newtrk_jt0p53_max)
			{
			  hnpion_jt0p53->AddBinContent(1, pthatw);
                          hnpion_jt0p53_[nchbin]->AddBinContent(1, pthatw);

			  if(trgtrk_id == 211)
                            {
                              hnpion_jt0p53_pm->AddBinContent(1, pthatw);
                              hnpion_jt0p53_pm_[nchbin]->AddBinContent(1, pthatw);
                            }
                          else if(trgtrk_id == -211)
                            {
                              hnpion_jt0p53_pm->AddBinContent(2, pthatw);
                              hnpion_jt0p53_pm_[nchbin]->AddBinContent(2, pthatw);
                            }
			}
		    }
		  if(TMath::Abs(trgtrk_id) == 321) //kaon
		    {
		      hkaon_pt->Fill(old_trgtrk_pt, pthatw);
                      hkaon_eta->Fill(old_trgtrk_eta, pthatw);
                      hkaon_phi->Fill(old_trgtrk_phi, pthatw);
                      hkaon_pt_jetaxis->Fill(new_trgtrk_pt, pthatw);
                      hkaon_eta_jetaxis->Fill(new_trgtrk_eta, pthatw);
                      hkaon_phi_jetaxis->Fill(new_trgtrk_phi, pthatw);

		      if(trgtrk_id == 321)
                        {
                          hkaon_pt_p->Fill(old_trgtrk_pt, pthatw);
                          hkaon_eta_p->Fill(old_trgtrk_eta, pthatw);
                          hkaon_phi_p->Fill(old_trgtrk_phi, pthatw);
                          hkaon_pt_jetaxis_p->Fill(new_trgtrk_pt, pthatw);
                          hkaon_eta_jetaxis_p->Fill(new_trgtrk_eta, pthatw);
                          hkaon_phi_jetaxis_p->Fill(new_trgtrk_phi, pthatw);
                        }
                      else if(trgtrk_id == -321)
                        {
                          hkaon_pt_m->Fill(old_trgtrk_pt, pthatw);
                          hkaon_eta_m->Fill(old_trgtrk_eta, pthatw);
                          hkaon_phi_m->Fill(old_trgtrk_phi, pthatw);
                          hkaon_pt_jetaxis_m->Fill(new_trgtrk_pt, pthatw);
                          hkaon_eta_jetaxis_m->Fill(new_trgtrk_eta, pthatw);
                          hkaon_phi_jetaxis_m->Fill(new_trgtrk_phi, pthatw);
                        }

		      if(new_trgtrk_pt >= newtrk_jt03_min && new_trgtrk_pt <= newtrk_jt03_max)
                        {
                          hnkaon_jt03->AddBinContent(1, pthatw);
                          hnkaon_jt03_[nchbin]->AddBinContent(1, pthatw);

			  if(trgtrk_id == 321)
                            {
                              hnkaon_jt03_pm->AddBinContent(1, pthatw);
                              hnkaon_jt03_pm_[nchbin]->AddBinContent(1, pthatw);
                            }
                          else if(trgtrk_id == -321)
                            {
                              hnkaon_jt03_pm->AddBinContent(2, pthatw);
                              hnkaon_jt03_pm_[nchbin]->AddBinContent(2, pthatw);
                            }
                        }
                      if(new_trgtrk_pt >= newtrk_jt0p33_min && new_trgtrk_pt <= newtrk_jt0p33_max)
                        {
                          hnkaon_jt0p33->AddBinContent(1, pthatw);
                          hnkaon_jt0p33_[nchbin]->AddBinContent(1, pthatw);

			  if(trgtrk_id == 321)
                            {
                              hnkaon_jt0p33_pm->AddBinContent(1, pthatw);
                              hnkaon_jt0p33_pm_[nchbin]->AddBinContent(1, pthatw);
                            }
                          else if(trgtrk_id == -321)
                            {
                              hnkaon_jt0p33_pm->AddBinContent(2, pthatw);
                              hnkaon_jt0p33_pm_[nchbin]->AddBinContent(2, pthatw);
                            }

                        }
                      if(new_trgtrk_pt >= newtrk_jt0p53_min && new_trgtrk_pt <= newtrk_jt0p53_max)
			{
                          hnkaon_jt0p53->AddBinContent(1, pthatw);
                          hnkaon_jt0p53_[nchbin]->AddBinContent(1, pthatw);

			  if(trgtrk_id == 321)
                            {
                              hnkaon_jt0p53_pm->AddBinContent(1, pthatw);
                              hnkaon_jt0p53_pm_[nchbin]->AddBinContent(1, pthatw);
                            }
                          else if(trgtrk_id == -321)
                            {
                              hnkaon_jt0p53_pm->AddBinContent(2, pthatw);
                              hnkaon_jt0p53_pm_[nchbin]->AddBinContent(2, pthatw);
                            }
                        }
		    }
		  if(TMath::Abs(trgtrk_id) == 2212) //ptoton
		    {
		      hproton_pt->Fill(old_trgtrk_pt, pthatw);
                      hproton_eta->Fill(old_trgtrk_eta, pthatw);
                      hproton_phi->Fill(old_trgtrk_phi, pthatw);
                      hproton_pt_jetaxis->Fill(new_trgtrk_pt, pthatw);
                      hproton_eta_jetaxis->Fill(new_trgtrk_eta, pthatw);
                      hproton_phi_jetaxis->Fill(new_trgtrk_phi, pthatw);

		      if(trgtrk_id == 2212)
                        {
                          hproton_pt_p->Fill(old_trgtrk_pt, pthatw);
                          hproton_eta_p->Fill(old_trgtrk_eta, pthatw);
                          hproton_phi_p->Fill(old_trgtrk_phi, pthatw);
                          hproton_pt_jetaxis_p->Fill(new_trgtrk_pt, pthatw);
                          hproton_eta_jetaxis_p->Fill(new_trgtrk_eta, pthatw);
                          hproton_phi_jetaxis_p->Fill(new_trgtrk_phi, pthatw);
                        }
                      else if(trgtrk_id == -2212)
                        {
                          hproton_pt_m->Fill(old_trgtrk_pt, pthatw);
                          hproton_eta_m->Fill(old_trgtrk_eta, pthatw);
                          hproton_phi_m->Fill(old_trgtrk_phi, pthatw);
                          hproton_pt_jetaxis_m->Fill(new_trgtrk_pt, pthatw);
                          hproton_eta_jetaxis_m->Fill(new_trgtrk_eta, pthatw);
                          hproton_phi_jetaxis_m->Fill(new_trgtrk_phi, pthatw);
                        }

		      if(new_trgtrk_pt >= newtrk_jt03_min && new_trgtrk_pt <= newtrk_jt03_max)
                        {
                          hnproton_jt03->AddBinContent(1, pthatw);
                          hnproton_jt03_[nchbin]->AddBinContent(1, pthatw);

			  if(trgtrk_id == 2212)
                            {
                              hnproton_jt03_pm->AddBinContent(1, pthatw);
                              hnproton_jt03_pm_[nchbin]->AddBinContent(1, pthatw);
                            }
                          else if(trgtrk_id == -2212)
                            {
                              hnproton_jt03_pm->AddBinContent(2, pthatw);
                              hnproton_jt03_pm_[nchbin]->AddBinContent(2, pthatw);
                            }
                        }
                      if(new_trgtrk_pt >= newtrk_jt0p33_min && new_trgtrk_pt <= newtrk_jt0p33_max)
                        {
                          hnproton_jt0p33->AddBinContent(1, pthatw);
                          hnproton_jt0p33_[nchbin]->AddBinContent(1, pthatw);

			  if(trgtrk_id == 2212)
                            {
                              hnproton_jt0p33_pm->AddBinContent(1, pthatw);
                              hnproton_jt0p33_pm_[nchbin]->AddBinContent(1, pthatw);
                            }
                          else if(trgtrk_id == -2212)
                            {
                              hnproton_jt0p33_pm->AddBinContent(2, pthatw);
                              hnproton_jt0p33_pm_[nchbin]->AddBinContent(2, pthatw);
                            }
                        }
                      if(new_trgtrk_pt >= newtrk_jt0p53_min && new_trgtrk_pt <= newtrk_jt0p53_max)
			{
                          hnproton_jt0p53->AddBinContent(1, pthatw);
                          hnproton_jt0p53_[nchbin]->AddBinContent(1, pthatw);

			  if(trgtrk_id == 2212)
                            {
                              hnproton_jt0p53_pm->AddBinContent(1, pthatw);
                              hnproton_jt0p53_pm_[nchbin]->AddBinContent(1, pthatw);
                            }
                          else if(trgtrk_id == -2212)
                            {
                              hnproton_jt0p53_pm->AddBinContent(2, pthatw);
                              hnproton_jt0p53_pm_[nchbin]->AddBinContent(2, pthatw);
			    }
                        }
		    }
		  if(TMath::Abs(trgtrk_id) == 3122) //lambda
		    {
		      hlambda_pt->Fill(old_trgtrk_pt, pthatw);
                      hlambda_eta->Fill(old_trgtrk_eta, pthatw);
                      hlambda_phi->Fill(old_trgtrk_phi, pthatw);
                      hlambda_pt_jetaxis->Fill(new_trgtrk_pt, pthatw);
                      hlambda_eta_jetaxis->Fill(new_trgtrk_eta, pthatw);
                      hlambda_phi_jetaxis->Fill(new_trgtrk_phi, pthatw);

		      if(trgtrk_id == 3122)
                        {
                          hlambda_pt_p->Fill(old_trgtrk_pt, pthatw);
                          hlambda_eta_p->Fill(old_trgtrk_eta, pthatw);
                          hlambda_phi_p->Fill(old_trgtrk_phi, pthatw);
                          hlambda_pt_jetaxis_p->Fill(new_trgtrk_pt, pthatw);
                          hlambda_eta_jetaxis_p->Fill(new_trgtrk_eta, pthatw);
                          hlambda_phi_jetaxis_p->Fill(new_trgtrk_phi, pthatw);
                        }
                      else if(trgtrk_id == -3122)
                        {
                          hlambda_pt_m->Fill(old_trgtrk_pt, pthatw);
                          hlambda_eta_m->Fill(old_trgtrk_eta, pthatw);
                          hlambda_phi_m->Fill(old_trgtrk_phi, pthatw);
                          hlambda_pt_jetaxis_m->Fill(new_trgtrk_pt, pthatw);
                          hlambda_eta_jetaxis_m->Fill(new_trgtrk_eta, pthatw);
                          hlambda_phi_jetaxis_m->Fill(new_trgtrk_phi, pthatw);
                        }

		      if(new_trgtrk_pt >= newtrk_jt03_min && new_trgtrk_pt <= newtrk_jt03_max)
                        {
                          hnlambda_jt03->AddBinContent(1, pthatw);
                          hnlambda_jt03_[nchbin]->AddBinContent(1, pthatw);

			  if(trgtrk_id == 3122)
                            {
                              hnlambda_jt03_pm->AddBinContent(1, pthatw);
                              hnlambda_jt03_pm_[nchbin]->AddBinContent(1, pthatw);
                            }
                          else if(trgtrk_id == -3122)
                            {
                              hnlambda_jt03_pm->AddBinContent(2, pthatw);
                              hnlambda_jt03_pm_[nchbin]->AddBinContent(2, pthatw);
                            }
                        }
                      if(new_trgtrk_pt >= newtrk_jt0p33_min && new_trgtrk_pt <= newtrk_jt0p33_max)
                        {
                          hnlambda_jt0p33->AddBinContent(1, pthatw);
                          hnlambda_jt0p33_[nchbin]->AddBinContent(1, pthatw);

			  if(trgtrk_id == 3122)
                            {
                              hnlambda_jt0p33_pm->AddBinContent(1, pthatw);
                              hnlambda_jt0p33_pm_[nchbin]->AddBinContent(1, pthatw);
                            }
                          else if(trgtrk_id == -3122)
                            {
                              hnlambda_jt0p33_pm->AddBinContent(2, pthatw);
                              hnlambda_jt0p33_pm_[nchbin]->AddBinContent(2, pthatw);
                            }
                        }
                      if(new_trgtrk_pt >= newtrk_jt0p53_min && new_trgtrk_pt <= newtrk_jt0p53_max)
			{
                          hnlambda_jt0p53->AddBinContent(1, pthatw);
                          hnlambda_jt0p53_[nchbin]->AddBinContent(1, pthatw);
			  if(trgtrk_id == 3122)
                            {
                              hnlambda_jt0p53_pm->AddBinContent(1, pthatw);
                              hnlambda_jt0p53_pm_[nchbin]->AddBinContent(1, pthatw);
                            }
                          else if(trgtrk_id == -3122)
                            {
                              hnlambda_jt0p53_pm->AddBinContent(2, pthatw);
                              hnlambda_jt0p53_pm_[nchbin]->AddBinContent(2, pthatw);
                            }
                        }
		    }
	
		} // eta condition
	      
	      if(TMath::Abs(trgtrk_charge) > 0 )
		{
		  if(TMath::Abs(new_trgtrk_eta) <= newtrk_eta_max)
		    {
		      hchtrk_pt->Fill(old_trgtrk_pt, pthatw);
		      hchtrk_eta->Fill(old_trgtrk_eta, pthatw);
		      hchtrk_phi->Fill(old_trgtrk_phi, pthatw);
		      hchtrk_pt_jetaxis->Fill(new_trgtrk_pt, pthatw);
		      hchtrk_eta_jetaxis->Fill(new_trgtrk_eta, pthatw);
		      hchtrk_phi_jetaxis->Fill(new_trgtrk_phi, pthatw);

		      if(trgtrk_charge > 0)
			{
			  hchtrk_pt_p->Fill(old_trgtrk_pt, pthatw);
			  hchtrk_eta_p->Fill(old_trgtrk_eta, pthatw);
			  hchtrk_phi_p->Fill(old_trgtrk_phi, pthatw);
			  hchtrk_pt_jetaxis_p->Fill(new_trgtrk_pt, pthatw);
			  hchtrk_eta_jetaxis_p->Fill(new_trgtrk_eta, pthatw);
			  hchtrk_phi_jetaxis_p->Fill(new_trgtrk_phi, pthatw);
			}
		      else if(trgtrk_charge < 0)
			{
			  hchtrk_pt_m->Fill(old_trgtrk_pt, pthatw);
			  hchtrk_eta_m->Fill(old_trgtrk_eta, pthatw);
			  hchtrk_phi_m->Fill(old_trgtrk_phi, pthatw);
			  hchtrk_pt_jetaxis_m->Fill(new_trgtrk_pt, pthatw);
			  hchtrk_eta_jetaxis_m->Fill(new_trgtrk_eta, pthatw);
			  hchtrk_phi_jetaxis_m->Fill(new_trgtrk_phi, pthatw);
			}
		      
		      if(new_trgtrk_pt >= newtrk_jt03_min && new_trgtrk_pt <= newtrk_jt03_max)
			{
			  hnchtrk_jt03->AddBinContent(1, pthatw);
			  hnchtrk_jt03_[nchbin]->AddBinContent(1, pthatw);

			  if(trgtrk_charge > 0)
			    {
			      hnchtrk_jt03_pm->AddBinContent(1, pthatw);
			      hnchtrk_jt03_pm_[nchbin]->AddBinContent(1, pthatw);
			    }
			  else if(trgtrk_charge < 0)
			    {
			      hnchtrk_jt03_pm->AddBinContent(2, pthatw);
			      hnchtrk_jt03_pm_[nchbin]->AddBinContent(2, pthatw);
			    }
			}
		      if(new_trgtrk_pt >= newtrk_jt0p33_min && new_trgtrk_pt <= newtrk_jt0p33_max)
			{
			  hnchtrk_jt0p33->AddBinContent(1, pthatw);
			  hnchtrk_jt0p33_[nchbin]->AddBinContent(1, pthatw);

			  if(trgtrk_charge > 0)
                            {
                              hnchtrk_jt0p33_pm->AddBinContent(1, pthatw);
                              hnchtrk_jt0p33_pm_[nchbin]->AddBinContent(1, pthatw);
                            }
                          else if(trgtrk_charge < 0)
                            {
                              hnchtrk_jt0p33_pm->AddBinContent(2, pthatw);
                              hnchtrk_jt0p33_pm_[nchbin]->AddBinContent(2, pthatw);
			    }
			}
		      if(new_trgtrk_pt >= newtrk_jt0p53_min && new_trgtrk_pt <= newtrk_jt0p53_max)
			{
			  hnchtrk_jt0p53->AddBinContent(1, pthatw);
			  hnchtrk_jt0p53_[nchbin]->AddBinContent(1, pthatw);

			  if(trgtrk_charge > 0)
                            {
                              hnchtrk_jt0p53_pm->AddBinContent(1, pthatw);
                              hnchtrk_jt0p53_pm_[nchbin]->AddBinContent(1, pthatw);
                            }
                          else if(trgtrk_charge < 0)
                            {
                              hnchtrk_jt0p53_pm->AddBinContent(2, pthatw);
                              hnchtrk_jt0p53_pm_[nchbin]->AddBinContent(2, pthatw);
                            }
			}
		    } // trk eta condition
		} // charge condition
	      */

	      for(int jtrk = 0; jtrk < (int)jet_newtrk_vec[ievt][ijet].size(); jtrk++) // associate trk loop
		{
		  if(itrk == jtrk) continue; // to avoid auto correlation
		  
		  TVector3 old_assotrk_vec = jet_oldtrk_vec[ievt][ijet][jtrk];
		  double old_assotrk_pt = old_assotrk_vec.Pt();
		  double old_assotrk_eta = old_assotrk_vec.Eta();
		  double old_assotrk_phi = old_assotrk_vec.Phi();
		  TVector3 new_assotrk_vec = jet_newtrk_vec[ievt][ijet][jtrk];
		  double new_assotrk_pt = new_assotrk_vec.Pt();
		  double new_assotrk_eta = new_assotrk_vec.Eta();
		  double new_assotrk_phi = new_assotrk_vec.Phi();
		  int assotrk_charge = jet_trk_charge[ievt][ijet][jtrk];
		  int assotrk_id = jet_trk_id[ievt][ijet][jtrk];

		  //if(new_assotrk_pt > newtrk_jt03_max) continue; // asso pt cut
			      
		  double deta_jetaxis = new_trgtrk_eta - new_assotrk_eta;
		  double dphi_jetaxis = new_trgtrk_phi - new_assotrk_phi;
		  double dphi2_jetaxis = new_assotrk_phi - new_trgtrk_phi;

		  if(dphi_jetaxis > 1.5*TMath::Pi())
		    {
		      dphi_jetaxis = dphi_jetaxis - 2.0*TMath::Pi();
		    }
		  else if(dphi_jetaxis < -0.5*TMath::Pi())
		    {
		      dphi_jetaxis = dphi_jetaxis + 2.0*TMath::Pi();
		    }
		  
		  if(dphi2_jetaxis > 1.5*TMath::Pi())
		    {
		      dphi2_jetaxis = dphi2_jetaxis - 2.0*TMath::Pi();
		    }
		  else if(dphi2_jetaxis < -0.5*TMath::Pi())
		    {
		      dphi2_jetaxis = dphi2_jetaxis + 2.0*TMath::Pi();
		    }

		  if(TMath::Abs(trgtrk_charge) > 0 && TMath::Abs(assotrk_charge) > 0)
		    {
		      if(TMath::Abs(new_trgtrk_eta) <= newtrk_eta_max && TMath::Abs(new_assotrk_eta) <= newtrk_eta_max)
			{
			  if(new_trgtrk_pt > 0. && new_assotrk_pt > 0.)
			    {
			      double NewTrk_Signal_1[4] = {TMath::Abs(deta_jetaxis), dphi_jetaxis, (double)jetptbin, (double)jet_flavour};
			      double NewTrk_Signal_2[4] = {-TMath::Abs(deta_jetaxis), dphi_jetaxis, (double)jetptbin, (double)jet_flavour};
			      double NewTrk_Signal_3[4] = {TMath::Abs(deta_jetaxis), dphi2_jetaxis, (double)jetptbin, (double)jet_flavour};
			      double NewTrk_Signal_4[4] = {-TMath::Abs(deta_jetaxis), dphi2_jetaxis, (double)jetptbin, (double)jet_flavour};

			      hNewTrk_Signal->Fill(NewTrk_Signal_1, pthatw/4.);
			      hNewTrk_Signal->Fill(NewTrk_Signal_2, pthatw/4.);
			      hNewTrk_Signal->Fill(NewTrk_Signal_3, pthatw/4.);
			      hNewTrk_Signal->Fill(NewTrk_Signal_4, pthatw/4.);

			      if(trgtrk_charge > 0 && assotrk_charge > 0)
				{
				  hNewTrk_Signal_pp->Fill(NewTrk_Signal_1, pthatw/4.);
				  hNewTrk_Signal_pp->Fill(NewTrk_Signal_2, pthatw/4.);
				  hNewTrk_Signal_pp->Fill(NewTrk_Signal_3, pthatw/4.);
				  hNewTrk_Signal_pp->Fill(NewTrk_Signal_4, pthatw/4.);
				}

			      if(trgtrk_charge < 0 && assotrk_charge < 0)
				{
				  hNewTrk_Signal_mm->Fill(NewTrk_Signal_1, pthatw/4.);
				  hNewTrk_Signal_mm->Fill(NewTrk_Signal_2, pthatw/4.);
				  hNewTrk_Signal_mm->Fill(NewTrk_Signal_3, pthatw/4.);
				  hNewTrk_Signal_mm->Fill(NewTrk_Signal_4, pthatw/4.);
				}

			      if(trgtrk_charge > 0 && assotrk_charge < 0)
                                {
                                  hNewTrk_Signal_pm->Fill(NewTrk_Signal_1, pthatw/4.);
                                  hNewTrk_Signal_pm->Fill(NewTrk_Signal_2, pthatw/4.);
                                  hNewTrk_Signal_pm->Fill(NewTrk_Signal_3, pthatw/4.);
                                  hNewTrk_Signal_pm->Fill(NewTrk_Signal_4, pthatw/4.);
                                }

			      if(trgtrk_charge < 0 && assotrk_charge > 0)
                                {
                                  hNewTrk_Signal_mp->Fill(NewTrk_Signal_1, pthatw/4.);
                                  hNewTrk_Signal_mp->Fill(NewTrk_Signal_2, pthatw/4.);
                                  hNewTrk_Signal_mp->Fill(NewTrk_Signal_3, pthatw/4.);
                                  hNewTrk_Signal_mp->Fill(NewTrk_Signal_4, pthatw/4.);
                                }
			    } //pt condition
			} // eta condition
		    } // charge condition
		  /*
		  if(TMath::Abs(trgtrk_charge) > 0 && TMath::Abs(assotrk_charge) > 0)
		    {
		      if(TMath::Abs(new_trgtrk_eta) <= newtrk_eta_max && TMath::Abs(new_assotrk_eta) <= newtrk_eta_max)
			{
			  if((new_trgtrk_pt >= newtrk_jt03_min && new_trgtrk_pt <= newtrk_jt03_max) && (new_assotrk_pt >= newtrk_jt03_min && new_assotrk_pt <= newtrk_jt03_max))
			    {      
			      hsignal_newchtrk_jt03->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
			      hsignal_newchtrk_jt03->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
			      hsignal_newchtrk_jt03->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
			      hsignal_newchtrk_jt03->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
			      
			      hsignal_newchtrk_jt03_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newchtrk_jt03_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newchtrk_jt03_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                              hsignal_newchtrk_jt03_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

			      if(trgtrk_charge > 0 && assotrk_charge > 0)
				{
				  hsignal_newchtrk_jt03_pp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				  hsignal_newchtrk_jt03_pp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				  hsignal_newchtrk_jt03_pp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				  hsignal_newchtrk_jt03_pp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newchtrk_jt03_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt03_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt03_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt03_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_charge < 0 && assotrk_charge < 0)
				{
				  hsignal_newchtrk_jt03_mm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt03_mm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt03_mm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt03_mm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newchtrk_jt03_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt03_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt03_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt03_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_charge > 0 && assotrk_charge < 0)
				{
				  hsignal_newchtrk_jt03_pm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt03_pm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt03_pm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt03_pm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newchtrk_jt03_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt03_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt03_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt03_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_charge < 0 && assotrk_charge > 0)
				{
				  hsignal_newchtrk_jt03_mp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt03_mp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt03_mp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt03_mp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newchtrk_jt03_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt03_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt03_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt03_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			    }
			  if((new_trgtrk_pt >= newtrk_jt0p33_min && new_trgtrk_pt <= newtrk_jt0p33_max) && (new_assotrk_pt >= newtrk_jt0p33_min && new_assotrk_pt <= newtrk_jt0p33_max))
			    {
			      hsignal_newchtrk_jt0p33->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newchtrk_jt0p33->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newchtrk_jt0p33->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                              hsignal_newchtrk_jt0p33->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

			      hsignal_newchtrk_jt0p33_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newchtrk_jt0p33_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newchtrk_jt0p33_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                              hsignal_newchtrk_jt0p33_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

			      if(trgtrk_charge > 0 && assotrk_charge > 0)
				{
				  hsignal_newchtrk_jt0p33_pp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				  hsignal_newchtrk_jt0p33_pp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				  hsignal_newchtrk_jt0p33_pp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				  hsignal_newchtrk_jt0p33_pp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newchtrk_jt0p33_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p33_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p33_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p33_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_charge < 0 && assotrk_charge < 0)
				{
				  hsignal_newchtrk_jt0p33_mm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p33_mm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p33_mm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p33_mm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newchtrk_jt0p33_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p33_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p33_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p33_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_charge > 0 && assotrk_charge < 0)
				{
				  hsignal_newchtrk_jt0p33_pm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p33_pm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p33_pm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p33_pm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newchtrk_jt0p33_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p33_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p33_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p33_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_charge < 0 && assotrk_charge > 0)
				{
				  hsignal_newchtrk_jt0p33_mp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p33_mp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p33_mp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p33_mp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newchtrk_jt0p33_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p33_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p33_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p33_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			    }
			  if((new_trgtrk_pt >= newtrk_jt0p53_min && new_trgtrk_pt <= newtrk_jt0p53_max) && (new_assotrk_pt >= newtrk_jt0p53_min && new_assotrk_pt <= newtrk_jt0p53_max))
			    {
			      hsignal_newchtrk_jt0p53->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newchtrk_jt0p53->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newchtrk_jt0p53->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                              hsignal_newchtrk_jt0p53->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

			      hsignal_newchtrk_jt0p53_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newchtrk_jt0p53_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newchtrk_jt0p53_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                              hsignal_newchtrk_jt0p53_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

			      if(trgtrk_charge > 0 && assotrk_charge > 0)
				{
				  hsignal_newchtrk_jt0p53_pp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				  hsignal_newchtrk_jt0p53_pp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				  hsignal_newchtrk_jt0p53_pp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				  hsignal_newchtrk_jt0p53_pp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newchtrk_jt0p53_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p53_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p53_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p53_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_charge < 0 && assotrk_charge < 0)
				{
				  hsignal_newchtrk_jt0p53_mm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p53_mm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p53_mm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p53_mm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newchtrk_jt0p53_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p53_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p53_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p53_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_charge > 0 && assotrk_charge < 0)
				{
				  hsignal_newchtrk_jt0p53_pm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p53_pm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p53_pm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p53_pm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newchtrk_jt0p53_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p53_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p53_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p53_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_charge < 0 && assotrk_charge > 0)
				{
				  hsignal_newchtrk_jt0p53_mp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p53_mp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p53_mp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p53_mp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newchtrk_jt0p53_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p53_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p53_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newchtrk_jt0p53_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			    }			      
			} // eta condition
		    } // charge condition
		
		  if(TMath::Abs(new_trgtrk_eta) <= newtrk_eta_max && TMath::Abs(new_assotrk_eta) <= newtrk_eta_max)
		    {
		      if((new_trgtrk_pt >= newtrk_jt03_min && new_trgtrk_pt <= newtrk_jt03_max) && (new_assotrk_pt >= newtrk_jt03_min && new_assotrk_pt <= newtrk_jt03_max))
			{
			  if(TMath::Abs(trgtrk_id) == 211 && TMath::Abs(assotrk_id) == 211)
			    {
			      hsignal_newpion_jt03->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newpion_jt03->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newpion_jt03->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                              hsignal_newpion_jt03->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

                              hsignal_newpion_jt03_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newpion_jt03_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newpion_jt03_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                              hsignal_newpion_jt03_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

			      if(trgtrk_id == 211 && assotrk_id == 211)
				{
				  hsignal_newpion_jt03_pp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				  hsignal_newpion_jt03_pp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				  hsignal_newpion_jt03_pp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				  hsignal_newpion_jt03_pp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newpion_jt03_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt03_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt03_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt03_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_id == -211 && assotrk_id == -211)
				{
				  hsignal_newpion_jt03_mm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt03_mm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt03_mm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt03_mm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newpion_jt03_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt03_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt03_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt03_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_id == 211 && assotrk_id == -211)
				{
				  hsignal_newpion_jt03_pm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt03_pm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt03_pm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt03_pm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newpion_jt03_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt03_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt03_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt03_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_id == -211 && assotrk_id == 211)
				{
				  hsignal_newpion_jt03_mp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt03_mp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt03_mp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt03_mp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newpion_jt03_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt03_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt03_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt03_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			    }
			  if(TMath::Abs(trgtrk_id) == 321 && TMath::Abs(assotrk_id) == 321)
			    {
			      hsignal_newkaon_jt03->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newkaon_jt03->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newkaon_jt03->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                              hsignal_newkaon_jt03->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

                              hsignal_newkaon_jt03_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newkaon_jt03_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newkaon_jt03_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                              hsignal_newkaon_jt03_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

			      if(trgtrk_id == 321 && assotrk_id == 321)
				{
				  hsignal_newkaon_jt03_pp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				  hsignal_newkaon_jt03_pp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				  hsignal_newkaon_jt03_pp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				  hsignal_newkaon_jt03_pp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newkaon_jt03_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt03_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt03_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt03_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_id == -321 && assotrk_id == -321)
				{
				  hsignal_newkaon_jt03_mm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt03_mm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt03_mm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt03_mm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newkaon_jt03_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt03_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt03_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt03_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_id == 321 && assotrk_id == -321)
				{
				  hsignal_newkaon_jt03_pm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt03_pm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt03_pm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt03_pm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newkaon_jt03_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt03_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt03_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt03_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_id == -321 && assotrk_id == 321)
				{
				  hsignal_newkaon_jt03_mp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt03_mp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt03_mp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt03_mp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newkaon_jt03_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt03_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt03_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt03_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			    }
			  if(TMath::Abs(trgtrk_id) == 2212 && TMath::Abs(assotrk_id) == 2212)
			    {
			      hsignal_newproton_jt03->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newproton_jt03->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newproton_jt03->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                              hsignal_newproton_jt03->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

                              hsignal_newproton_jt03_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newproton_jt03_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newproton_jt03_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                              hsignal_newproton_jt03_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

			      if(trgtrk_id == 2212 && assotrk_id == 2212)
				{
				  hsignal_newproton_jt03_pp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				  hsignal_newproton_jt03_pp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				  hsignal_newproton_jt03_pp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				  hsignal_newproton_jt03_pp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newproton_jt03_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt03_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt03_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt03_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_id == -2212 && assotrk_id == -2212)
				{
				  hsignal_newproton_jt03_mm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt03_mm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt03_mm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt03_mm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newproton_jt03_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt03_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt03_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt03_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_id == 2212 && assotrk_id == -2212)
				{
				  hsignal_newproton_jt03_pm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt03_pm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt03_pm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt03_pm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newproton_jt03_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt03_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt03_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt03_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_id == -2212 && assotrk_id == 2212)
				{
				  hsignal_newproton_jt03_mp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt03_mp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt03_mp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt03_mp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newproton_jt03_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt03_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt03_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt03_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			    }
			  if(TMath::Abs(trgtrk_id) == 3122 && TMath::Abs(assotrk_id) == 3122)
			    {
			      hsignal_newlambda_jt03->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newlambda_jt03->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newlambda_jt03->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                              hsignal_newlambda_jt03->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

                              hsignal_newlambda_jt03_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newlambda_jt03_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newlambda_jt03_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                              hsignal_newlambda_jt03_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

			      if(trgtrk_id == 3122 && assotrk_id == 3122)
				{
				  hsignal_newlambda_jt03_pp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				  hsignal_newlambda_jt03_pp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				  hsignal_newlambda_jt03_pp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				  hsignal_newlambda_jt03_pp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newlambda_jt03_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt03_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt03_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt03_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_id == -3122 && assotrk_id == -3122)
				{
				  hsignal_newlambda_jt03_mm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt03_mm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt03_mm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt03_mm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newlambda_jt03_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt03_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt03_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt03_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_id == 3122 && assotrk_id == -3122)
				{
				  hsignal_newlambda_jt03_pm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt03_pm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt03_pm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt03_pm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newlambda_jt03_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt03_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt03_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt03_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_id == -3122 && assotrk_id == 3122)
				{
				  hsignal_newlambda_jt03_mp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt03_mp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt03_mp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt03_mp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newlambda_jt03_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt03_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt03_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt03_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			    }
			}
		      
		      if((new_trgtrk_pt >= newtrk_jt0p33_min && new_trgtrk_pt <= newtrk_jt0p33_max) && (new_assotrk_pt >= newtrk_jt0p33_min && new_assotrk_pt <= newtrk_jt0p33_max))
			{
			  if(TMath::Abs(trgtrk_id) == 211 && TMath::Abs(assotrk_id) == 211)
			    {
			      hsignal_newpion_jt0p33->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newpion_jt0p33->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newpion_jt0p33->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                              hsignal_newpion_jt0p33->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

                              hsignal_newpion_jt0p33_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newpion_jt0p33_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newpion_jt0p33_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                              hsignal_newpion_jt0p33_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

			      if(trgtrk_id == 211 && assotrk_id == 211)
				{
				  hsignal_newpion_jt0p33_pp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				  hsignal_newpion_jt0p33_pp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				  hsignal_newpion_jt0p33_pp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				  hsignal_newpion_jt0p33_pp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newpion_jt0p33_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p33_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p33_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p33_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_id == -211 && assotrk_id == -211)
				{
				  hsignal_newpion_jt0p33_mm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p33_mm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p33_mm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p33_mm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newpion_jt0p33_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p33_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p33_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p33_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_id == 211 && assotrk_id == -211)
				{
				  hsignal_newpion_jt0p33_pm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p33_pm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p33_pm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p33_pm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newpion_jt0p33_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p33_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p33_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p33_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_id == -211 && assotrk_id == 211)
				{
				  hsignal_newpion_jt0p33_mp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p33_mp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p33_mp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p33_mp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newpion_jt0p33_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p33_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p33_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p33_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			    }
			  if(TMath::Abs(trgtrk_id) == 321 && TMath::Abs(assotrk_id) == 321)
			    {
			      hsignal_newkaon_jt0p33->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newkaon_jt0p33->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newkaon_jt0p33->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                              hsignal_newkaon_jt0p33->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

                              hsignal_newkaon_jt0p33_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newkaon_jt0p33_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newkaon_jt0p33_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                              hsignal_newkaon_jt0p33_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

			      if(trgtrk_id == 321 && assotrk_id == 321)
				{
				  hsignal_newkaon_jt0p33_pp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				  hsignal_newkaon_jt0p33_pp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				  hsignal_newkaon_jt0p33_pp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				  hsignal_newkaon_jt0p33_pp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newkaon_jt0p33_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p33_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p33_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p33_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_id == -321 && assotrk_id == -321)
				{
				  hsignal_newkaon_jt0p33_mm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p33_mm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p33_mm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p33_mm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newkaon_jt0p33_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p33_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p33_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p33_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_id == 321 && assotrk_id == -321)
				{
				  hsignal_newkaon_jt0p33_pm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p33_pm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p33_pm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p33_pm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newkaon_jt0p33_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p33_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p33_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p33_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_id == -321 && assotrk_id == 321)
				{
				  hsignal_newkaon_jt0p33_mp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p33_mp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p33_mp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p33_mp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newkaon_jt0p33_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p33_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p33_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p33_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			    }
			  if(TMath::Abs(trgtrk_id) == 2212 && TMath::Abs(assotrk_id) == 2212)
			    {
			      hsignal_newproton_jt0p33->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newproton_jt0p33->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newproton_jt0p33->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                              hsignal_newproton_jt0p33->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

                              hsignal_newproton_jt0p33_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newproton_jt0p33_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newproton_jt0p33_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                              hsignal_newproton_jt0p33_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

			      if(trgtrk_id == 2212 && assotrk_id == 2212)
				{
				  hsignal_newproton_jt0p33_pp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				  hsignal_newproton_jt0p33_pp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				  hsignal_newproton_jt0p33_pp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				  hsignal_newproton_jt0p33_pp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newproton_jt0p33_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p33_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p33_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p33_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_id == -2212 && assotrk_id == -2212)
				{
				  hsignal_newproton_jt0p33_mm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p33_mm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p33_mm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p33_mm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newproton_jt0p33_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p33_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p33_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p33_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_id == 2212 && assotrk_id == -2212)
				{
				  hsignal_newproton_jt0p33_pm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p33_pm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p33_pm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p33_pm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newproton_jt0p33_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p33_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p33_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p33_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_id == -2212 && assotrk_id == 2212)
				{
				  hsignal_newproton_jt0p33_mp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p33_mp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p33_mp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p33_mp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newproton_jt0p33_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p33_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p33_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p33_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			    }
			  if(TMath::Abs(trgtrk_id) == 3122 && TMath::Abs(assotrk_id) == 3122)
			    {
			      hsignal_newlambda_jt0p33->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newlambda_jt0p33->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newlambda_jt0p33->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                              hsignal_newlambda_jt0p33->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

                              hsignal_newlambda_jt0p33_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newlambda_jt0p33_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newlambda_jt0p33_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                              hsignal_newlambda_jt0p33_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

			      if(trgtrk_id == 3122 && assotrk_id == 3122)
				{
				  hsignal_newlambda_jt0p33_pp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				  hsignal_newlambda_jt0p33_pp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				  hsignal_newlambda_jt0p33_pp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				  hsignal_newlambda_jt0p33_pp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newlambda_jt0p33_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p33_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p33_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p33_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_id == -3122 && assotrk_id == -3122)
				{
				  hsignal_newlambda_jt0p33_mm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p33_mm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p33_mm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p33_mm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newlambda_jt0p33_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p33_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p33_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p33_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_id == 3122 && assotrk_id == -3122)
				{
				  hsignal_newlambda_jt0p33_pm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p33_pm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p33_pm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p33_pm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newlambda_jt0p33_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p33_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p33_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p33_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_id == -3122 && assotrk_id == 3122)
				{
				  hsignal_newlambda_jt0p33_mp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p33_mp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p33_mp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p33_mp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newlambda_jt0p33_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p33_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p33_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p33_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			    }
			}
		      if((new_trgtrk_pt >= newtrk_jt0p53_min && new_trgtrk_pt <= newtrk_jt0p53_max) && (new_assotrk_pt >= newtrk_jt0p53_min && new_assotrk_pt <= newtrk_jt0p53_max))
			{
			  if(TMath::Abs(trgtrk_id) == 211 && TMath::Abs(assotrk_id) == 211)
			    {
			      hsignal_newpion_jt0p53->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newpion_jt0p53->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newpion_jt0p53->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                              hsignal_newpion_jt0p53->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

                              hsignal_newpion_jt0p53_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newpion_jt0p53_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newpion_jt0p53_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                              hsignal_newpion_jt0p53_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

			      if(trgtrk_id == 211 && assotrk_id == 211)
				{
				  hsignal_newpion_jt0p53_pp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				  hsignal_newpion_jt0p53_pp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				  hsignal_newpion_jt0p53_pp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				  hsignal_newpion_jt0p53_pp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newpion_jt0p53_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p53_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p53_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p53_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_id == -211 && assotrk_id == -211)
				{
				  hsignal_newpion_jt0p53_mm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p53_mm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p53_mm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p53_mm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newpion_jt0p53_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p53_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p53_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p53_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_id == 211 && assotrk_id == -211)
				{
				  hsignal_newpion_jt0p53_pm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p53_pm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p53_pm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p53_pm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newpion_jt0p53_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p53_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p53_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p53_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_id == -211 && assotrk_id == 211)
				{
				  hsignal_newpion_jt0p53_mp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p53_mp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p53_mp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p53_mp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newpion_jt0p53_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p53_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p53_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newpion_jt0p53_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			    }
			  if(TMath::Abs(trgtrk_id) == 321 && TMath::Abs(assotrk_id) == 321)
			    {
			      hsignal_newkaon_jt0p53->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newkaon_jt0p53->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newkaon_jt0p53->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                              hsignal_newkaon_jt0p53->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

                              hsignal_newkaon_jt0p53_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newkaon_jt0p53_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newkaon_jt0p53_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                              hsignal_newkaon_jt0p53_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

			      if(trgtrk_id == 321 && assotrk_id == 321)
				{
				  hsignal_newkaon_jt0p53_pp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				  hsignal_newkaon_jt0p53_pp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				  hsignal_newkaon_jt0p53_pp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				  hsignal_newkaon_jt0p53_pp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newkaon_jt0p53_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p53_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p53_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p53_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_id == -321 && assotrk_id == -321)
				{
				  hsignal_newkaon_jt0p53_mm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p53_mm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p53_mm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p53_mm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newkaon_jt0p53_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p53_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p53_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p53_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_id == 321 && assotrk_id == -321)
				{
				  hsignal_newkaon_jt0p53_pm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p53_pm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p53_pm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p53_pm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newkaon_jt0p53_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p53_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p53_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p53_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_id == -321 && assotrk_id == 321)
				{
				  hsignal_newkaon_jt0p53_mp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p53_mp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p53_mp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p53_mp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newkaon_jt0p53_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p53_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p53_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newkaon_jt0p53_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			    }
			  if(TMath::Abs(trgtrk_id) == 2212 && TMath::Abs(assotrk_id) == 2212)
			    {
			      hsignal_newproton_jt0p53->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newproton_jt0p53->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newproton_jt0p53->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                              hsignal_newproton_jt0p53->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

                              hsignal_newproton_jt0p53_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newproton_jt0p53_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newproton_jt0p53_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                              hsignal_newproton_jt0p53_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

			      if(trgtrk_id == 2212 && assotrk_id == 2212)
				{
				  hsignal_newproton_jt0p53_pp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				  hsignal_newproton_jt0p53_pp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				  hsignal_newproton_jt0p53_pp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				  hsignal_newproton_jt0p53_pp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newproton_jt0p53_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p53_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p53_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p53_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_id == -2212 && assotrk_id == -2212)
				{
				  hsignal_newproton_jt0p53_mm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p53_mm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p53_mm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p53_mm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newproton_jt0p53_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p53_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p53_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p53_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_id == 2212 && assotrk_id == -2212)
				{
				  hsignal_newproton_jt0p53_pm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p53_pm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p53_pm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p53_pm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newproton_jt0p53_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p53_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p53_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p53_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_id == -2212 && assotrk_id == 2212)
				{
				  hsignal_newproton_jt0p53_mp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p53_mp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p53_mp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p53_mp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newproton_jt0p53_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p53_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p53_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newproton_jt0p53_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			    }
			  if(TMath::Abs(trgtrk_id) == 3122 && TMath::Abs(assotrk_id) == 3122)
			    {
			      hsignal_newlambda_jt0p53->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newlambda_jt0p53->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newlambda_jt0p53->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                              hsignal_newlambda_jt0p53->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

                              hsignal_newlambda_jt0p53_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newlambda_jt0p53_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                              hsignal_newlambda_jt0p53_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                              hsignal_newlambda_jt0p53_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

			      if(trgtrk_id == 3122 && assotrk_id == 3122)
				{
				  hsignal_newlambda_jt0p53_pp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				  hsignal_newlambda_jt0p53_pp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				  hsignal_newlambda_jt0p53_pp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				  hsignal_newlambda_jt0p53_pp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newlambda_jt0p53_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p53_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p53_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p53_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_id == -3122 && assotrk_id == -3122)
				{
				  hsignal_newlambda_jt0p53_mm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p53_mm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p53_mm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p53_mm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newlambda_jt0p53_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p53_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p53_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p53_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_id == 3122 && assotrk_id == -3122)
				{
				  hsignal_newlambda_jt0p53_pm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p53_pm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p53_pm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p53_pm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newlambda_jt0p53_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p53_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p53_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p53_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			      if(trgtrk_id == -3122 && assotrk_id == 3122)
				{
				  hsignal_newlambda_jt0p53_mp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p53_mp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p53_mp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p53_mp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				  hsignal_newlambda_jt0p53_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p53_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p53_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
                                  hsignal_newlambda_jt0p53_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				}
			    }
			}
		    } // eta condition
		  */

		} // asso trk loop
	    } // trg trk loop end
	  njets++;
	} // jet loop end
      if(njets > 0)
	{
	  hnjets_afterCut->Fill(njets);
	}
    } // event loop end
  std::cout<<endl;
  std::cout<<"~~~~~end signal correlation~~~~~~~~"<<std::endl;
} // function loop

void mixing_corr(std::vector<float> pthatw_vec, std::vector<int> processID_vec, std::vector<std::vector<TVector3>> jet_vec, std::vector<std::vector<int>> jet_flavour_vec, std::vector<std::vector<std::vector<TVector3>>> jet_oldtrk_vec, std::vector<std::vector<std::vector<TVector3>>> jet_newtrk_vec, std::vector<std::vector<std::vector<TVector3>>> jet_oldchtrk_vec, std::vector<std::vector<std::vector<TVector3>>> jet_newchtrk_vec, std::vector<std::vector<std::vector<int>>> jet_trk_charge, std::vector<std::vector<std::vector<int>>> jet_trk_id)
{
  std::cout<<endl;
  std::cout<<"~~~~~start mixed event correlation~~~~~~~~"<<std::endl;
  
  for(int ievt = 0; ievt < (int)jet_newtrk_vec.size(); ievt++) // 1st event loop
    {
      if(ievt%10000 == 0) std::cout<<ievt<<"  events running for mixing correlation of total events "<< jet_newtrk_vec.size() <<std::endl;
      
      double pthatw = pthatw_vec[ievt];
      int subprocessID = processID_vec[ievt];

      // mixing algorithm
      int mixstart = ievt+1;
      int mixend   = (int)jet_newtrk_vec.size();

      if(mixstart > (0.7*(jet_newtrk_vec.size())))
	{
	  int mixstart = ievt;
	  int mixend   = (int)jet_newtrk_vec.size();
	}

      int nmix = 0;
      for(int jevt = mixstart; jevt < mixend; jevt++) // 2nd event loop
	{
	  if(ievt == jevt) continue;
	  
	  nmix++;
	  if(nmix > bkgFactor) break;

	  for(int ijet = 0; ijet < (int)jet_newtrk_vec[ievt].size(); ijet++) // 1st jet loop
	    {
	      TVector3 jetVec = jet_vec[ievt][ijet];
	      double jet_pt = jetVec.Pt();
	      double jet_eta = jetVec.Eta();
	      double jet_phi = jetVec.Phi();
	      int jet_flavour = jet_flavour_vec[ievt][ijet];
	      int nchtrk_jet = jet_newchtrk_vec[ievt][ijet].size();
	      int ntrk_jet = jet_newtrk_vec[ievt][ijet].size();
	      
	      // extract nchtrk bin
	      int nchbin  = hnch_bin_hist->FindBin(nchtrk_jet) - 1;
	      
	      //apply jet cut
	      if(jet_pt < jet_pt_min || jet_pt > jet_pt_max || TMath::Abs(jet_eta) > jet_eta_max) continue;

	      // extract jet pt bin
	      int jetptbin  = hjetpt_bin_hist->FindBin(jet_pt) - 1;
	      
	      for(int jjet = 0; jjet < (int)jet_newtrk_vec[jevt].size(); jjet++) // 2nd jet loop
		{
		  TVector3 jetVec2 = jet_vec[jevt][jjet];
		  double jet_pt2 = jetVec.Pt();
		  double jet_eta2 = jetVec.Eta();
		  double jet_phi2 = jetVec.Phi();
		  int nchtrk_jet2 = jet_newchtrk_vec[jevt][jjet].size();
		  int ntrk_jet2 = jet_newtrk_vec[jevt][jjet].size();

		  // apply jet cut
		  if(jet_pt2 < jet_pt_min || jet_pt2 > jet_pt_max || TMath::Abs(jet_eta2) > jet_eta_max) continue;
		  
		  for(int itrk = 0; itrk < (int)jet_newtrk_vec[ievt][ijet].size(); itrk++) // trigger trk loop
		    {
		      TVector3 old_trgtrk_vec = jet_oldtrk_vec[ievt][ijet][itrk];
		      double old_trgtrk_pt = old_trgtrk_vec.Pt();
		      double old_trgtrk_eta = old_trgtrk_vec.Eta();
		      double old_trgtrk_phi = old_trgtrk_vec.Phi();
		      TVector3 new_trgtrk_vec = jet_newtrk_vec[ievt][ijet][itrk];
		      double new_trgtrk_pt = new_trgtrk_vec.Pt();
		      double new_trgtrk_eta = new_trgtrk_vec.Eta();
		      double new_trgtrk_phi = new_trgtrk_vec.Phi();
		      int trgtrk_charge = jet_trk_charge[ievt][ijet][itrk];
		      int trgtrk_id = jet_trk_id[ievt][ijet][itrk];

		      //if(new_trgtrk_pt > newtrk_jt03_max) continue; // trg pt cut
		      
		      for(int jtrk = 0; jtrk < (int)jet_newtrk_vec[jevt][jjet].size(); jtrk++) // associate trk loop
			{
			  TVector3 old_assotrk_vec = jet_oldtrk_vec[jevt][jjet][jtrk];
			  double old_assotrk_pt = old_assotrk_vec.Pt();
			  double old_assotrk_eta = old_assotrk_vec.Eta();
			  double old_assotrk_phi = old_assotrk_vec.Phi();
			  TVector3 new_assotrk_vec = jet_newtrk_vec[jevt][jjet][jtrk];
			  double new_assotrk_pt = new_assotrk_vec.Pt();
			  double new_assotrk_eta = new_assotrk_vec.Eta();
			  double new_assotrk_phi = new_assotrk_vec.Phi();
			  int assotrk_charge = jet_trk_charge[jevt][jjet][jtrk];
			  int assotrk_id = jet_trk_id[jevt][jjet][jtrk];

			  //if(new_assotrk_pt > newtrk_jt03_max) continue; // asso pt cut
			  
			  double deta_jetaxis = new_trgtrk_eta - new_assotrk_eta;
			  double dphi_jetaxis = new_trgtrk_phi - new_assotrk_phi;
			  double dphi2_jetaxis = new_assotrk_phi - new_trgtrk_phi;
			  
			  if(dphi_jetaxis > 1.5*TMath::Pi())
			    {
			      dphi_jetaxis = dphi_jetaxis - 2.0*TMath::Pi();
			    }
			  else if(dphi_jetaxis < -0.5*TMath::Pi())
			    {
			      dphi_jetaxis = dphi_jetaxis + 2.0*TMath::Pi();
			    }
			  
			  if(dphi2_jetaxis > 1.5*TMath::Pi())
			    {
			      dphi2_jetaxis = dphi2_jetaxis - 2.0*TMath::Pi();
			    }
			  else if(dphi2_jetaxis < -0.5*TMath::Pi())
			    {
			      dphi2_jetaxis = dphi2_jetaxis + 2.0*TMath::Pi();
			    }
			  
			  if(TMath::Abs(trgtrk_charge) > 0 && TMath::Abs(assotrk_charge) > 0)
			    {
			      if(TMath::Abs(new_trgtrk_eta) <= newtrk_eta_max && TMath::Abs(new_assotrk_eta) <= newtrk_eta_max)
				{
				  if(new_trgtrk_pt > 0. && new_assotrk_pt > 0.)
				    {
				      double NewTrk_Mixing_1[4] = {TMath::Abs(deta_jetaxis), dphi_jetaxis, (double)jetptbin, (double)jet_flavour};
				      double NewTrk_Mixing_2[4] = {-TMath::Abs(deta_jetaxis), dphi_jetaxis, (double)jetptbin, (double)jet_flavour};
				      double NewTrk_Mixing_3[4] = {TMath::Abs(deta_jetaxis), dphi2_jetaxis, (double)jetptbin, (double)jet_flavour};
				      double NewTrk_Mixing_4[4] = {-TMath::Abs(deta_jetaxis), dphi2_jetaxis, (double)jetptbin, (double)jet_flavour};
				      
				      hNewTrk_Mixing->Fill(NewTrk_Mixing_1, pthatw/4.);
				      hNewTrk_Mixing->Fill(NewTrk_Mixing_2, pthatw/4.);
				      hNewTrk_Mixing->Fill(NewTrk_Mixing_3, pthatw/4.);
				      hNewTrk_Mixing->Fill(NewTrk_Mixing_4, pthatw/4.);
				      
				      if(trgtrk_charge > 0 && assotrk_charge > 0)
					{
					  hNewTrk_Mixing_pp->Fill(NewTrk_Mixing_1, pthatw/4.);
					  hNewTrk_Mixing_pp->Fill(NewTrk_Mixing_2, pthatw/4.);
					  hNewTrk_Mixing_pp->Fill(NewTrk_Mixing_3, pthatw/4.);
					  hNewTrk_Mixing_pp->Fill(NewTrk_Mixing_4, pthatw/4.);
					}
				      
				      if(trgtrk_charge < 0 && assotrk_charge < 0)
					{
					  hNewTrk_Mixing_mm->Fill(NewTrk_Mixing_1, pthatw/4.);
					  hNewTrk_Mixing_mm->Fill(NewTrk_Mixing_2, pthatw/4.);
					  hNewTrk_Mixing_mm->Fill(NewTrk_Mixing_3, pthatw/4.);
					  hNewTrk_Mixing_mm->Fill(NewTrk_Mixing_4, pthatw/4.);
					}
				      
				      if(trgtrk_charge > 0 && assotrk_charge < 0)
					{
					  hNewTrk_Mixing_pm->Fill(NewTrk_Mixing_1, pthatw/4.);
					  hNewTrk_Mixing_pm->Fill(NewTrk_Mixing_2, pthatw/4.);
					  hNewTrk_Mixing_pm->Fill(NewTrk_Mixing_3, pthatw/4.);
					  hNewTrk_Mixing_pm->Fill(NewTrk_Mixing_4, pthatw/4.);
					}
				      
				      if(trgtrk_charge < 0 && assotrk_charge > 0)
					{
					  hNewTrk_Mixing_mp->Fill(NewTrk_Mixing_1, pthatw/4.);
					  hNewTrk_Mixing_mp->Fill(NewTrk_Mixing_2, pthatw/4.);
					  hNewTrk_Mixing_mp->Fill(NewTrk_Mixing_3, pthatw/4.);
					  hNewTrk_Mixing_mp->Fill(NewTrk_Mixing_4, pthatw/4.);
					}
				    } //pt condition
				} // eta condition
			    } // charge condition
			  
			  /*		  
			  if(TMath::Abs(trgtrk_charge) > 0 && TMath::Abs(assotrk_charge) > 0)
			    {
			      if(TMath::Abs(new_trgtrk_eta) <= newtrk_eta_max && TMath::Abs(new_assotrk_eta) <= newtrk_eta_max)
				{
				  if((new_trgtrk_pt >= newtrk_jt03_min && new_trgtrk_pt <= newtrk_jt03_max) && (new_assotrk_pt >= newtrk_jt03_min && new_assotrk_pt <= newtrk_jt03_max))
				    {
				      hmixing_newchtrk_jt03->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newchtrk_jt03->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newchtrk_jt03->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      hmixing_newchtrk_jt03->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      
				      hmixing_newchtrk_jt03_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newchtrk_jt03_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newchtrk_jt03_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      hmixing_newchtrk_jt03_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      
				      if(trgtrk_charge > 0 && assotrk_charge > 0)
					{
					  hmixing_newchtrk_jt03_pp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt03_pp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt03_pp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt03_pp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newchtrk_jt03_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt03_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt03_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt03_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_charge < 0 && assotrk_charge < 0)
					{
					  hmixing_newchtrk_jt03_mm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt03_mm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt03_mm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt03_mm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newchtrk_jt03_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt03_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt03_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt03_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_charge > 0 && assotrk_charge < 0)
					{
					  hmixing_newchtrk_jt03_pm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt03_pm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt03_pm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt03_pm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newchtrk_jt03_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt03_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt03_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt03_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_charge < 0 && assotrk_charge > 0)
					{
					  hmixing_newchtrk_jt03_mp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt03_mp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt03_mp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt03_mp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newchtrk_jt03_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt03_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt03_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt03_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				    }
				  if((new_trgtrk_pt >= newtrk_jt0p33_min && new_trgtrk_pt <= newtrk_jt0p33_max) && (new_assotrk_pt >= newtrk_jt0p33_min && new_assotrk_pt <= newtrk_jt0p33_max))
				    {
				      hmixing_newchtrk_jt0p33->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newchtrk_jt0p33->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newchtrk_jt0p33->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      hmixing_newchtrk_jt0p33->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      
				      hmixing_newchtrk_jt0p33_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newchtrk_jt0p33_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newchtrk_jt0p33_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      hmixing_newchtrk_jt0p33_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      if(trgtrk_charge > 0 && assotrk_charge > 0)
					{
					  hmixing_newchtrk_jt0p33_pp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p33_pp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p33_pp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p33_pp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newchtrk_jt0p33_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p33_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p33_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p33_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_charge < 0 && assotrk_charge < 0)
					{
					  hmixing_newchtrk_jt0p33_mm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p33_mm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p33_mm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p33_mm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newchtrk_jt0p33_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p33_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p33_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p33_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_charge > 0 && assotrk_charge < 0)
					{
					  hmixing_newchtrk_jt0p33_pm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p33_pm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p33_pm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p33_pm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newchtrk_jt0p33_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p33_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p33_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p33_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_charge < 0 && assotrk_charge > 0)
					{
					  hmixing_newchtrk_jt0p33_mp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p33_mp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p33_mp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p33_mp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newchtrk_jt0p33_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p33_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p33_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p33_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				    }
				  if((new_trgtrk_pt >= newtrk_jt0p53_min && new_trgtrk_pt <= newtrk_jt0p53_max) && (new_assotrk_pt >= newtrk_jt0p53_min && new_assotrk_pt <= newtrk_jt0p53_max))
				    {
				      hmixing_newchtrk_jt0p53->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newchtrk_jt0p53->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newchtrk_jt0p53->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      hmixing_newchtrk_jt0p53->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      
				      hmixing_newchtrk_jt0p53_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newchtrk_jt0p53_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newchtrk_jt0p53_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      hmixing_newchtrk_jt0p53_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      
				      if(trgtrk_charge > 0 && assotrk_charge > 0)
					{
					  hmixing_newchtrk_jt0p53_pp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p53_pp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p53_pp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p53_pp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newchtrk_jt0p53_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p53_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p53_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p53_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_charge < 0 && assotrk_charge < 0)
					{
					  hmixing_newchtrk_jt0p53_mm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p53_mm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p53_mm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p53_mm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newchtrk_jt0p53_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p53_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p53_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p53_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_charge > 0 && assotrk_charge < 0)
					{
					  hmixing_newchtrk_jt0p53_pm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p53_pm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p53_pm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p53_pm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newchtrk_jt0p53_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p53_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p53_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p53_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_charge < 0 && assotrk_charge > 0)
					{
					  hmixing_newchtrk_jt0p53_mp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p53_mp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p53_mp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p53_mp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newchtrk_jt0p53_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p53_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p53_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newchtrk_jt0p53_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				    }
				} // eta condition
			    } // charge condition
			
			  if(TMath::Abs(new_trgtrk_eta) <= newtrk_eta_max && TMath::Abs(new_assotrk_eta) <= newtrk_eta_max)
			    {
			      if((new_trgtrk_pt >= newtrk_jt03_min && new_trgtrk_pt <= newtrk_jt03_max) && (new_assotrk_pt >= newtrk_jt03_min && new_assotrk_pt <= newtrk_jt03_max))
				{
				  if(TMath::Abs(trgtrk_id) == 211 && TMath::Abs(assotrk_id) == 211)
				    {
				      hmixing_newpion_jt03->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newpion_jt03->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newpion_jt03->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      hmixing_newpion_jt03->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      
				      hmixing_newpion_jt03_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newpion_jt03_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newpion_jt03_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      hmixing_newpion_jt03_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				      if(trgtrk_id == 211 && assotrk_id == 211)
					{
					  hmixing_newpion_jt03_pp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt03_pp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt03_pp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newpion_jt03_pp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newpion_jt03_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt03_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt03_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newpion_jt03_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_id == -211 && assotrk_id == -211)
					{
					  hmixing_newpion_jt03_mm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt03_mm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt03_mm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newpion_jt03_mm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newpion_jt03_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt03_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt03_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newpion_jt03_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_id == 211 && assotrk_id == -211)
					{
					  hmixing_newpion_jt03_pm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt03_pm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt03_pm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newpion_jt03_pm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newpion_jt03_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt03_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt03_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newpion_jt03_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_id == -211 && assotrk_id == 211)
					{
					  hmixing_newpion_jt03_mp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt03_mp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt03_mp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newpion_jt03_mp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newpion_jt03_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt03_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt03_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newpion_jt03_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				    }
				  if(TMath::Abs(trgtrk_id) == 321 && TMath::Abs(assotrk_id) == 321)
				    {
				      hmixing_newkaon_jt03->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newkaon_jt03->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newkaon_jt03->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      hmixing_newkaon_jt03->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      
				      hmixing_newkaon_jt03_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newkaon_jt03_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newkaon_jt03_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      hmixing_newkaon_jt03_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				      if(trgtrk_id == 321 && assotrk_id == 321)
					{
					  hmixing_newkaon_jt03_pp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt03_pp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt03_pp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt03_pp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newkaon_jt03_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt03_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt03_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt03_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_id == -321 && assotrk_id == -321)
					{
					  hmixing_newkaon_jt03_mm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt03_mm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt03_mm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt03_mm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newkaon_jt03_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt03_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt03_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt03_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_id == 321 && assotrk_id == -321)
					{
					  hmixing_newkaon_jt03_pm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt03_pm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt03_pm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt03_pm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newkaon_jt03_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt03_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt03_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt03_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_id == -321 && assotrk_id == 321)
					{
					  hmixing_newkaon_jt03_mp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt03_mp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt03_mp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt03_mp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newkaon_jt03_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt03_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt03_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt03_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				    }
				  if(TMath::Abs(trgtrk_id) == 2212 && TMath::Abs(assotrk_id) == 2212)
				    {
				      hmixing_newproton_jt03->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newproton_jt03->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newproton_jt03->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      hmixing_newproton_jt03->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      
				      hmixing_newproton_jt03_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newproton_jt03_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newproton_jt03_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      hmixing_newproton_jt03_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      
				      if(trgtrk_id == 2212 && assotrk_id == 2212)
					{
					  hmixing_newproton_jt03_pp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt03_pp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt03_pp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newproton_jt03_pp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newproton_jt03_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt03_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt03_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newproton_jt03_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_id == -2212 && assotrk_id == -2212)
					{
					  hmixing_newproton_jt03_mm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt03_mm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt03_mm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newproton_jt03_mm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newproton_jt03_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt03_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt03_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newproton_jt03_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_id == 2212 && assotrk_id == -2212)
					{
					  hmixing_newproton_jt03_pm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt03_pm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt03_pm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newproton_jt03_pm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newproton_jt03_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt03_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt03_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newproton_jt03_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_id == -2212 && assotrk_id == 2212)
					{
					  hmixing_newproton_jt03_mp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt03_mp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt03_mp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newproton_jt03_mp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newproton_jt03_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt03_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt03_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newproton_jt03_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				    }
				  if(TMath::Abs(trgtrk_id) == 3122 && TMath::Abs(assotrk_id) == 3122)
				    {
				      hmixing_newlambda_jt03->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newlambda_jt03->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newlambda_jt03->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      hmixing_newlambda_jt03->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      
				      hmixing_newlambda_jt03_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newlambda_jt03_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newlambda_jt03_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      hmixing_newlambda_jt03_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				      if(trgtrk_id == 3122 && assotrk_id == 3122)
					{
					  hmixing_newlambda_jt03_pp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt03_pp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt03_pp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt03_pp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newlambda_jt03_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt03_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt03_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt03_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_id == -3122 && assotrk_id == -3122)
					{
					  hmixing_newlambda_jt03_mm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt03_mm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt03_mm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt03_mm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newlambda_jt03_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt03_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt03_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt03_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_id == 3122 && assotrk_id == -3122)
					{
					  hmixing_newlambda_jt03_pm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt03_pm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt03_pm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt03_pm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newlambda_jt03_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt03_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt03_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt03_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_id == -3122 && assotrk_id == 3122)
					{
					  hmixing_newlambda_jt03_mp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt03_mp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt03_mp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt03_mp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newlambda_jt03_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt03_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt03_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt03_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				    }
				}
			      
			      if((new_trgtrk_pt >= newtrk_jt0p33_min && new_trgtrk_pt <= newtrk_jt0p33_max) && (new_assotrk_pt >= newtrk_jt0p33_min && new_assotrk_pt <= newtrk_jt0p33_max))
				{
				  if(TMath::Abs(trgtrk_id) == 211 && TMath::Abs(assotrk_id) == 211)
				    {
				      hmixing_newpion_jt0p33->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newpion_jt0p33->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newpion_jt0p33->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      hmixing_newpion_jt0p33->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      
				      hmixing_newpion_jt0p33_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newpion_jt0p33_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newpion_jt0p33_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      hmixing_newpion_jt0p33_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				      if(trgtrk_id == 211 && assotrk_id == 211)
					{
					  hmixing_newpion_jt0p33_pp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p33_pp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p33_pp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p33_pp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newpion_jt0p33_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p33_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p33_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p33_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_id == -211 && assotrk_id == -211)
					{
					  hmixing_newpion_jt0p33_mm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p33_mm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p33_mm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p33_mm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newpion_jt0p33_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p33_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p33_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p33_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_id == 211 && assotrk_id == -211)
					{
					  hmixing_newpion_jt0p33_pm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p33_pm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p33_pm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p33_pm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newpion_jt0p33_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p33_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p33_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p33_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_id == -211 && assotrk_id == 211)
					{
					  hmixing_newpion_jt0p33_mp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p33_mp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p33_mp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p33_mp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newpion_jt0p33_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p33_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p33_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p33_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				    }
				  if(TMath::Abs(trgtrk_id) == 321 && TMath::Abs(assotrk_id) == 321)
				    {
				      hmixing_newkaon_jt0p33->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newkaon_jt0p33->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newkaon_jt0p33->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      hmixing_newkaon_jt0p33->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      
				      hmixing_newkaon_jt0p33_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newkaon_jt0p33_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newkaon_jt0p33_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      hmixing_newkaon_jt0p33_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				      if(trgtrk_id == 321 && assotrk_id == 321)
					{
					  hmixing_newkaon_jt0p33_pp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p33_pp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p33_pp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p33_pp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newkaon_jt0p33_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p33_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p33_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p33_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_id == -321 && assotrk_id == -321)
					{
					  hmixing_newkaon_jt0p33_mm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p33_mm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p33_mm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p33_mm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newkaon_jt0p33_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p33_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p33_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p33_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_id == 321 && assotrk_id == -321)
					{
					  hmixing_newkaon_jt0p33_pm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p33_pm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p33_pm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p33_pm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newkaon_jt0p33_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p33_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p33_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p33_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_id == -321 && assotrk_id == 321)
					{
					  hmixing_newkaon_jt0p33_mp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p33_mp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p33_mp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p33_mp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newkaon_jt0p33_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p33_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p33_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p33_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				    }
				  if(TMath::Abs(trgtrk_id) == 2212 && TMath::Abs(assotrk_id) == 2212)
				    {
				      hmixing_newproton_jt0p33->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newproton_jt0p33->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newproton_jt0p33->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      hmixing_newproton_jt0p33->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      
				      hmixing_newproton_jt0p33_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newproton_jt0p33_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newproton_jt0p33_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      hmixing_newproton_jt0p33_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				      if(trgtrk_id == 2212 && assotrk_id == 2212)
					{
					  hmixing_newproton_jt0p33_pp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p33_pp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p33_pp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p33_pp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newproton_jt0p33_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p33_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p33_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p33_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_id == -2212 && assotrk_id == -2212)
					{
					  hmixing_newproton_jt0p33_mm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p33_mm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p33_mm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p33_mm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newproton_jt0p33_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p33_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p33_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p33_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_id == 2212 && assotrk_id == -2212)
					{
					  hmixing_newproton_jt0p33_pm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p33_pm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p33_pm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p33_pm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newproton_jt0p33_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p33_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p33_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p33_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_id == -2212 && assotrk_id == 2212)
					{
					  hmixing_newproton_jt0p33_mp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p33_mp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p33_mp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p33_mp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newproton_jt0p33_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p33_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p33_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p33_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				    }
				  if(TMath::Abs(trgtrk_id) == 3122 && TMath::Abs(assotrk_id) == 3122)
				    {
				      hmixing_newlambda_jt0p33->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newlambda_jt0p33->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newlambda_jt0p33->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      hmixing_newlambda_jt0p33->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      
				      hmixing_newlambda_jt0p33_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newlambda_jt0p33_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newlambda_jt0p33_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      hmixing_newlambda_jt0p33_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				      if(trgtrk_id == 3122 && assotrk_id == 3122)
					{
					  hmixing_newlambda_jt0p33_pp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p33_pp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p33_pp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p33_pp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newlambda_jt0p33_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p33_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p33_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p33_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_id == -3122 && assotrk_id == -3122)
					{
					  hmixing_newlambda_jt0p33_mm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p33_mm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p33_mm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p33_mm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newlambda_jt0p33_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p33_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p33_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p33_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_id == 3122 && assotrk_id == -3122)
					{
					  hmixing_newlambda_jt0p33_pm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p33_pm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p33_pm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p33_pm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newlambda_jt0p33_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p33_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p33_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p33_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_id == -3122 && assotrk_id == 3122)
					{
					  hmixing_newlambda_jt0p33_mp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p33_mp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p33_mp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p33_mp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newlambda_jt0p33_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p33_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p33_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p33_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				    }
				}
			      if((new_trgtrk_pt >= newtrk_jt0p53_min && new_trgtrk_pt <= newtrk_jt0p53_max) && (new_assotrk_pt >= newtrk_jt0p53_min && new_assotrk_pt <= newtrk_jt0p53_max))
				{
				  if(TMath::Abs(trgtrk_id) == 211 && TMath::Abs(assotrk_id) == 211)
				    {
				      hmixing_newpion_jt0p53->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newpion_jt0p53->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newpion_jt0p53->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      hmixing_newpion_jt0p53->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      
				      hmixing_newpion_jt0p53_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newpion_jt0p53_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newpion_jt0p53_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      hmixing_newpion_jt0p53_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				      if(trgtrk_id == 211 && assotrk_id == 211)
					{
					  hmixing_newpion_jt0p53_pp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p53_pp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p53_pp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p53_pp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newpion_jt0p53_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p53_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p53_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p53_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_id == -211 && assotrk_id == -211)
					{
					  hmixing_newpion_jt0p53_mm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p53_mm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p53_mm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p53_mm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newpion_jt0p53_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p53_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p53_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p53_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_id == 211 && assotrk_id == -211)
					{
					  hmixing_newpion_jt0p53_pm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p53_pm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p53_pm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p53_pm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newpion_jt0p53_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p53_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p53_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p53_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_id == -211 && assotrk_id == 211)
					{
					  hmixing_newpion_jt0p53_mp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p53_mp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p53_mp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p53_mp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newpion_jt0p53_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p53_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p53_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newpion_jt0p53_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				    }
				  if(TMath::Abs(trgtrk_id) == 321 && TMath::Abs(assotrk_id) == 321)
				    {
				      hmixing_newkaon_jt0p53->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newkaon_jt0p53->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newkaon_jt0p53->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      hmixing_newkaon_jt0p53->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      
				      hmixing_newkaon_jt0p53_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newkaon_jt0p53_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newkaon_jt0p53_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      hmixing_newkaon_jt0p53_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				      if(trgtrk_id == 321 && assotrk_id == 321)
					{
					  hmixing_newkaon_jt0p53_pp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p53_pp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p53_pp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p53_pp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newkaon_jt0p53_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p53_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p53_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p53_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_id == -321 && assotrk_id == -321)
					{
					  hmixing_newkaon_jt0p53_mm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p53_mm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p53_mm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p53_mm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newkaon_jt0p53_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p53_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p53_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p53_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_id == 321 && assotrk_id == -321)
					{
					  hmixing_newkaon_jt0p53_pm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p53_pm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p53_pm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p53_pm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newkaon_jt0p53_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p53_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p53_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p53_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_id == -321 && assotrk_id == 321)
					{
					  hmixing_newkaon_jt0p53_mp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p53_mp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p53_mp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p53_mp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newkaon_jt0p53_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p53_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p53_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newkaon_jt0p53_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				    }
				  if(TMath::Abs(trgtrk_id) == 2212 && TMath::Abs(assotrk_id) == 2212)
				    {
				      hmixing_newproton_jt0p53->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newproton_jt0p53->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newproton_jt0p53->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      hmixing_newproton_jt0p53->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      
				      hmixing_newproton_jt0p53_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newproton_jt0p53_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newproton_jt0p53_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      hmixing_newproton_jt0p53_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				      if(trgtrk_id == 2212 && assotrk_id == 2212)
					{
					  hmixing_newproton_jt0p53_pp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p53_pp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p53_pp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p53_pp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newproton_jt0p53_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p53_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p53_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p53_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_id == -2212 && assotrk_id == -2212)
					{
					  hmixing_newproton_jt0p53_mm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p53_mm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p53_mm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p53_mm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newproton_jt0p53_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p53_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p53_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p53_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_id == 2212 && assotrk_id == -2212)
					{
					  hmixing_newproton_jt0p53_pm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p53_pm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p53_pm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p53_pm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newproton_jt0p53_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p53_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p53_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p53_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_id == -2212 && assotrk_id == 2212)
					{
					  hmixing_newproton_jt0p53_mp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p53_mp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p53_mp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p53_mp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newproton_jt0p53_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p53_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p53_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newproton_jt0p53_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				    }
				  if(TMath::Abs(trgtrk_id) == 3122 && TMath::Abs(assotrk_id) == 3122)
				    {
				      hmixing_newlambda_jt0p53->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newlambda_jt0p53->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newlambda_jt0p53->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      hmixing_newlambda_jt0p53->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      
				      hmixing_newlambda_jt0p53_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newlambda_jt0p53_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
				      hmixing_newlambda_jt0p53_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
				      hmixing_newlambda_jt0p53_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);

				      if(trgtrk_id == 3122 && assotrk_id == 3122)
					{
					  hmixing_newlambda_jt0p53_pp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p53_pp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p53_pp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p53_pp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newlambda_jt0p53_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p53_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p53_pp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p53_pp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_id == -3122 && assotrk_id == -3122)
					{
					  hmixing_newlambda_jt0p53_mm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p53_mm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p53_mm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p53_mm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newlambda_jt0p53_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p53_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p53_mm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p53_mm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_id == 3122 && assotrk_id == -3122)
					{
					  hmixing_newlambda_jt0p53_pm->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p53_pm->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p53_pm->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p53_pm->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newlambda_jt0p53_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p53_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p53_pm_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p53_pm_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				      if(trgtrk_id == -3122 && assotrk_id == 3122)
					{
					  hmixing_newlambda_jt0p53_mp->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p53_mp->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p53_mp->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p53_mp->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  
					  hmixing_newlambda_jt0p53_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p53_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p53_mp_[nchbin]->Fill(TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					  hmixing_newlambda_jt0p53_mp_[nchbin]->Fill(-TMath::Abs(deta_jetaxis), dphi2_jetaxis, pthatw/4.);
					}
				    }
				}
			    } // eta condition
			  */
			} // asso trk loop
		    } // trg trk loop end
		} // 1st jet loop end
	    } // 2nd jet loop end
	} // 2nd event loop end
    } // 1st event loop end
  std::cout<<endl;
  std::cout<<"~~~~~end mixed event correlation~~~~~~~~"<<std::endl;
} // function loop
      
void print_start()
{
  cout << endl;
  time_t init = time(0);
  char* init_time = ctime(&init); // convert now to string form                                        
  cout << "Starting at : " << init_time << endl;
  cout << "++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ " << endl;
  cout << endl;
}

void print_stop()
{
  time_t end = time(0);
  char* end_time = ctime(&end); // convert now to string form                                                   
  cout << endl;
  cout << "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" << endl;
  cout << endl;
  cout << "Stopping at : " << end_time << endl;
  cout << "+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++" << endl;
  cout << endl;
}
